@@ -1,18 +1,26 @@
 """
 .. note::
 
-    While modeltree is functional and well tested, it's in an early state of
-    its development. Backward incompatible api changes are possible. Feedback
-    and suggestions about the api are very welcome. Just open an issue on
-    github.
+    While modeltree is functional and well tested, it's in an early state of its
+    development. Backward incompatible api changes are possible. Feedback and
+    suggestions about the api are very welcome. Just open an issue on github.
+
+
+About
+-----
+Do you have a model layout with various relations and looking for a way to
+navigate it with ease? Then django-modeltree is what you are looking for. Build
+your modeltree in a single line and accessing related models and their objects
+in a elegant and performant way. No need for complex query building anymore.
+Give it a try...
 
 
 What is a ModelTree?
 --------------------
 
-A ModelTree describes a :class:`~django.db.models.Model` and all its
-recursive relations to other models. It is :class:`~anytree.node.node.Node`
-based, iterable, walkable, searchable and can be populated by
+A ModelTree describes a :class:`~django.db.models.Model` and all its recursive
+relations to other models. It is :class:`~anytree.node.node.Node` based,
+iterable, walkable, searchable and can be populated by
 :attr:`~.ModelTree.items`.
 
 Guess you have these models::
@@ -74,31 +82,54 @@ This is very easy. Simply pass in the model the tree should be rooted to::
 
     tree = ModelTree(ModelOne)
 
-Optionally you can pass in a queryset of your model. Every node then will be
-populated by items of the node's :attr:`~.ModelTree.model` that are related
-to these initial items by the direct or indirect relation of their models::
+Optionally you can pass in a queryset of your model::
 
     >>> items = ModelOne.objects.all()
     >>> tree = ModelTree(ModelOne, items)
 
-Guess you want all ModelFive items that are related to the ModelOne items
-with which you initiated your tree::
+You will be able then to recieve a queryset of related objects of the models of
+your modeltree by accessing the :attr:`~.ModelTree.items` attribute of each
+node::
 
     >>> items = ModelOne.objects.all()
     >>> tree = ModelTree(ModelOne, items)
-    >>> model_five_node = tree.get(filter=lambda n: n.model == ModelFive)
-    >>> len(model_five_node.items)
-    0
+    >>> len(tree['model_two']['model_three']['model_five'].items)
+    3
 
 See the :attr:`~.ModelTree.items` section for more information about how items
 are processed.
 
 
-What if I don't want to follow all model relations?
----------------------------------------------------
+How to navigate a modeltree?
+----------------------------
 
-You can easily adjust the way your tree is build up. Therefore overwrite one
-or more of the following class attributes in your ModelTree subclass:
+A model tree has a simple dict api implementation that allows you to access
+each child node by the name of its :attr:`~.ModelTree.field` attribute as
+key::
+
+    >>> tree = ModelTree(ModelOne)
+    >>> tree['model_two'].field.name
+    'model_two'
+
+Building your tree is a one-liner but gives you access to all related models and
+model objects at once - regardless of the direction or type of the involved
+relations::
+
+    >>> tree = ModelTree(ModelOne, ModelOne.objects.filter(pk__in=[1, 2, 3]))
+    >>> tree['model_two']['model_three'].model
+    <class 'testapp.models.ModelThree'>
+    >>> tree['model_two']['model_three'].items
+    <QuerySet [<ModelThree: ModelThree object (0)>, <ModelThree: ModelThree object (1)>]>
+
+For further options to access related tree nodes see :meth:`~.ModelTree.get`
+and :meth:`~.ModelTree.find`.
+
+
+How to customize my modeltree?
+------------------------------
+
+You can easily adjust the way your tree is build up. Therefore overwrite one or
+more of the following class attributes in your ModelTree subclass:
 
 * :attr:`~.ModelTree.MAX_DEPTH`
 * :attr:`~.ModelTree.FOLLOW_ACROSS_APPS`
@@ -167,7 +198,7 @@ class ModelTree(AnyNode):
                 └── model_five -> ModelFive
 
     In advance you can pass in some items of your model as a queryset. The
-    :attr:`.items` property of each node of your tree then reflects the itmes
+    :attr:`.items` property of each node of your tree then reflects the items
     that are derived from the initial items by the direct or indirect relations
     of their models.
 
@@ -181,7 +212,7 @@ class ModelTree(AnyNode):
         >>> tree = ModelTree(ModelOne, items)
         >>> model_four_node = tree.get(filter=lambda n: n.model == ModelFour)
         >>> len(model_four_node.items)
-        0
+        2
 
     :param model: model to start with
     :type model: :class:`~django.db.models.Model`
@@ -227,7 +258,7 @@ class ModelTree(AnyNode):
         ]
 
     .. note::
-    
+
         Generic relations using the contenttypes framework are not supported.
     """
 
@@ -255,6 +286,7 @@ class ModelTree(AnyNode):
         self._model = model
         self._field = field
         self._items = items
+        self._children_by_field = dict()
         self._build_tree()
 
     def __str__(self):
@@ -267,6 +299,15 @@ class ModelTree(AnyNode):
         classname = type(self).__name__
         return '{}(model={}, field={})'.format(classname, repr(self._model), repr(self._field))
 
+    def __contains__(self, __key):
+        return self._children_by_field.__contains__(__key)
+
+    def __getitem__(self, __key):
+        return self._children_by_field.__getitem__(__key)
+
+    def __iter__(self):
+        return self._children_by_field.__iter__()
+
     @property
     def model(self):
         """
@@ -298,7 +339,7 @@ class ModelTree(AnyNode):
     def field_path(self):
         """
         String describing the node's :attr:`~anytree.node.nodemixin.NodeMixin.path`
-        using the :attr:`.field_name` of nodes::
+        using the name of the node's :attr:`.field`::
 
             >>> node_four = list(tree.iterate())[3]
             >>> node_four.field_path
@@ -323,14 +364,14 @@ class ModelTree(AnyNode):
         it will be the :attr:`.items` attribute of the root node. All child
         nodes hold a queryset of elements that are derived of the initial one::
 
-            >>> items_one = ModelOne.objects.all().values_list('pk', flat=True)
+            >>> items_one = ModelOne.objects.all()
             >>> items_two = ModelTwo.objects.filter(
-            ...     model_one__pk__in=items_one).values_list('pk', flat=True)
+            ...     model_one__pk__in=items_one.values_list('pk', flat=True))
             >>> items_three = ModelThree.objects.filter(
-            ...     model_two__pk__in=items_two)
+            ...     model_two__pk__in=items_two.values_list('pk', flat=True))
             >>> tree = ModelTree(ModelOne, items_one)
             >>> node_three = tree.get('model_two__model_three')
-            >>> list(node_three.items) == list(items_three)
+            >>> set(node_three.items) == set(items_three)
             True
 
         Items of a node are lazy. Querysets are evaluated not until an items
@@ -355,7 +396,7 @@ class ModelTree(AnyNode):
         """
         return RenderTree(self)
 
-    def show(self, format='{node}', root_format='{node}'):
+    def show(self, format='{node}', root_format='{node}', with_items=False):
         """
         Print a tree. Each node will be rendered by using a format string which
         reference the node object by the key *node*::
@@ -368,47 +409,105 @@ class ModelTree(AnyNode):
                     ├── ModelThree.model_four -> ModelFour
                     └── ModelThree.model_five -> ModelFive
 
+        A format string referencing the :attr:`.field` attribute won't be
+        appropriate for the root node since it has no field. To adjust the way
+        the root node is rendered use the root_format keyword argument::
+
+            >>> tree = ModelTree(ModelOne)
+            >>> tree.show(root_format='<{node.model._meta.verbose_name}>')
+            <model one>
+            └── model_two -> ModelTwo
+                └── model_three -> ModelThree
+                    ├── model_four -> ModelFour
+                    └── model_five -> ModelFive
+
+        Use the show_items argument to render the tree with each node's items
+        listed below the node::
+
+            >>> tree = ModelTree(ModelTwo, ModelTwo.objects.filter(pk=4))
+            >>> tree.show(with_items=True)
+            ModelTwo
+            │ ~ ModelTwo object (4)
+            ├── model_one -> ModelOne
+            │     ~ ModelOne object (2)
+            └── model_three -> ModelThree
+                │ ~ ModelThree object (0)
+                ├── model_four -> ModelFour
+                │     ~ ModelFour object (0)
+                └── model_five -> ModelFive
+                      ~ ModelFive object (3)
+                      ~ ModelFive object (4)
+                      ~ ModelFive object (5)
+
         :param str format: format string to render a node object (optional)
         :param str root_format: format string to render the root node object (optional)
+        :param bool with_items: include the node's items (optional)
         """
-        for prefix, _, node in self.render():
+        for prefix, multiline_prefix, node in self.render():
             if root_format and node.is_root:
                 label = root_format.format(node=node)
             else:
                 label = format.format(node=node)
             print(f'{prefix}{label}')
+            if not node.items is None and with_items:
+                if node.is_leaf:
+                    item_prefix = '  ~ '
+                else:
+                    item_prefix = '│ ~ '
+                for item in node.items:
+                    print(f'{multiline_prefix}{item_prefix}{item}')
 
-    def get(self, field_path=None, filter=None):
+    def get(self, field_path=None, filter=None, **params):
         """
-        Either lookup a node by its :attr:`.field_path` or use a filter::
+        Either lookup a node by its :attr:`.field_path` or a filter or node
+        attributes::
 
             >>> tree = ModelTree(ModelOne)
             >>> tree.get('model_two') == tree.get(filter=lambda n: n.model == ModelTwo)
             True
+            >>> tree.get('model_two') == tree.get(model=ModelTwo)
+            True
 
         :param str field_path: a node's :attr:`.field_path` (optional)
-        :params: any key value pair referencing to a node's attribute
+        :param callable filter: callable taking a node and returning a boolean (optional)
+        :param \*\*params: any node attribute to use for a lookup (optional)
         :return: node object or None if no node matches the search parameters
         :rtype: :class:`~modeltree.ModelTree` or None
         :raises: :class:`~anytree.search.CountError`: if more than one node was found
         """
         if field_path:
             filter = lambda n: n.field_path == field_path
+        else:
+            filters = list()
+            for key, value in params.items():
+                filters.append(lambda n: getattr(n, key) == value)
+            if filter:
+                filters.append(filter)
+            filter = lambda n: all(f(n) for f in filters)
         return find(self, filter)
 
-    def find(self, filter):
+    def find(self, filter=None, **params):
         """
-        Find nodes using a filter::
+        Find nodes using a filter or node attributes::
 
             >>> tree = ModelTree(ModelOne)
+            >>> tree.find(lambda n: n.model == ModelTwo) == tree.find(model=ModelTwo)
+            True
             >>> len(tree.find(lambda n: n.relation_type == 'many_to_many'))
             2
             >>> len(tree.find(lambda n: type(n.field) == models.OneToOneRel))
             1
 
-        :params: any key value pair referencing to attributes of nodes
+        :param callable filter: callable taking a node and returning a boolean (optional)
+        :param \*\*params: any node attribute to use for a lookup (optional)
         :return: tuple of nodes
         """
+        filters = list()
+        for key, value in params.items():
+            filters.append(lambda n: getattr(n, key) == value)
+        if filter:
+            filters.append(filter)
+        filter = lambda n: all(f(n) for f in filters)
         return findall(self, filter)
 
     def iterate(self, by_level=False, by_grouped_level=False, maxlevel=None, has_items=False, filter=None):
@@ -457,7 +556,7 @@ class ModelTree(AnyNode):
             ...          return True
             ...       else:
             ...          return False
-            ... 
+            ...
             >>> tree = MyModelTree(ModelOne)
             >>> all(n.model._meta.app_label == 'testapp' for n in tree.iterate())
             True
@@ -522,4 +621,5 @@ class ModelTree(AnyNode):
         if self.depth < self.MAX_DEPTH:
             for field in self.model._meta.get_fields():
                 if self._follow_this_field(field):
-                    self.__class__(model=field.related_model, field=field, parent=self)
+                    node = self.__class__(model=field.related_model, field=field, parent=self)
+                    self._children_by_field[field.name] = node
